require "scraperapi/version"
require 'net/http'
require 'unirest'

module ScraperAPI
  class Client
    def initialize(api_key)
      @api_key = api_key
    end

    #
    #headers={},
    #    country_code=None,
    #    device_type=None,
    #    premium=None,
    #    render=None,
    #    session_number=None,
    #    autoparse=None,
    #    retry=3,
    #    timeout=60

    def get(url,
            headers: {},
            country_code: nil,
            device_type: nil,
            premium: false,
            render: false,
            session_number: nil,
            autoparse: false,
            retry_n: 3,
            timeout: 60)


      makeRequestWithMethod(url,
                            @api_key,
                            "GET",
                            headers: headers,
                            country_code: country_code,
                            device_type: device_type,
                            premium: premium,
                            render: render,
                            session_number: session_number,
                            autoparse: autoparse,
                            retry_n: retry_n,
                            timeout: timeout)

    end

    def post(url,
             headers: {},
             country_code: nil,
             device_type: nil,
             premium: false,
             render: false,
             session_number: nil,
             autoparse: false,
             retry_n: 3,
             timeout: 60,
             body: nil)

      makeRequestWithMethod(url,
                            @api_key,
                            "POST",
                            headers: headers,
                            country_code: country_code,
                            device_type: device_type,
                            premium: premium,
                            render: render,
                            session_number: session_number,
                            autoparse: autoparse,
                            retry_n: retry_n,
                            timeout: timeout,
                            body: body)
    end

    def put(url,
            headers: {},
            country_code: nil,
            device_type: nil,
            premium: false,
            render: false,
            session_number: nil,
            autoparse: false,
            retry_n: 3,
            timeout: 60,
            body: nil)

      makeRequestWithMethod(url,
                            @api_key,
                            "PUT",
                            headers: headers,
                            country_code: country_code,
                            device_type: device_type,
                            premium: premium,
                            render: render,
                            session_number: session_number,
                            autoparse: autoparse,
                            retry_n: retry_n,
                            timeout: timeout,
                            body: body)
    end

    def account()
      uri = URI.parse("https://api.scraperapi.com/account")
      uri.query = URI.decode(URI.encode_www_form({:api_key => @api_key}))
      return Unirest.get(uri.to_s).body
    end

    private

    def makeRequestWithMethod(url,
                              api_key,
                              method,
                              headers: {},
                              country_code: nil,
                              device_type: nil,
                              premium: false,
                              render: false,
                              session_number: nil,
                              autoparse: false,
                              retry_n: 3,
                              timeout: 60,
                              body: nil)


      params = {
          :api_key => api_key,
          :country_code => country_code,
          :device_type => device_type,
          :premium => premium,
          :render => render,
          :session_number => session_number,
          :autoparse => autoparse,
          :keep_headers => !headers.empty?,
          :url => url,
          :scraper_sdk => "ruby"
      }

      begin
        retries ||= 0
      Unirest.timeout(timeout)

      scraperapi_basepath = "https://api.scraperapi.com/"
      uri = URI.parse(scraperapi_basepath)
      uri.query = URI.decode(URI.encode_www_form(params.compact!))

      if method == "GET"
        return Unirest.get(uri.to_s, headers:headers)
      end
      if method == "POST"
        return Unirest.post(uri.to_s, headers:headers, parameters: body)
      end
      if method == "PUT"
        return Unirest.put(uri.to_s, headers:headers, parameters: body)
      end

      rescue
        retry if (retries += 1) <= retry_n
      end

    end

  end
end
